from datetime import timedelta

import pandas as pd
from django.utils.timezone import localdate
from django.utils.translation import gettext_lazy as _
from dynamic_preferences.registries import global_preferences_registry
from psycopg.types.range import TimestamptzRange
from wbcore import filters as wb_filters

from wbhuman_resources.models import AbsenceRequestPeriods, DayOffCalendar, Position


def current_year_date_range(*args, **kwargs):
    d = localdate()
    return TimestamptzRange(
        (localdate() - pd.tseries.offsets.Week(weekday=0)).date(),
        max((d + pd.tseries.offsets.YearEnd(1)).date(), d + timedelta(days=60)),
    )


def monday_of_current_week(*args, **kwargs):
    today = localdate()
    return today - timedelta(days=today.weekday())


def get_calendar_default(field, request, view, **kwargs) -> int | None:
    if (profile := request.user.profile) and (employee := getattr(profile, "human_resources", None)):
        return employee.calendar.id
    if calendar := global_preferences_registry.manager()["wbhuman_resources__employee_default_calendar"]:
        return calendar.id
    try:
        return DayOffCalendar.objects.first().id
    except AttributeError:
        return None


class AbsenceRequestPlannerFilter(wb_filters.FilterSet):
    calendar = wb_filters.ModelChoiceFilter(
        label=_("Calendar"),
        required=True,
        clearable=False,
        queryset=DayOffCalendar.objects.all(),
        endpoint=DayOffCalendar.get_representation_endpoint(),
        value_key=DayOffCalendar.get_representation_value_key(),
        label_key=DayOffCalendar.get_representation_label_key(),
        default=get_calendar_default,
        method=lambda queryset, label, value: queryset,
    )

    date = wb_filters.DateRangeFilter(
        label=_("Date Range"),
        method=lambda queryset, label, value: queryset,
        required=True,
        clearable=False,
        default=current_year_date_range,
    )
    only_employee_with_absence_periods = wb_filters.BooleanFilter(
        default=False,
        label=_("Only Employee With Absence periods"),
        method=lambda queryset, label, value: queryset,
    )

    position = wb_filters.ModelChoiceFilter(
        label=_("Position"),
        queryset=Position.objects.all(),
        endpoint=Position.get_representation_endpoint(),
        value_key=Position.get_representation_value_key(),
        label_key=Position.get_representation_label_key(),
        method=lambda queryset, label, value: queryset,
    )

    class Meta:
        model = AbsenceRequestPeriods
        fields = {}


class AbsenceTableFilter(AbsenceRequestPlannerFilter):
    date_gte = date_lte = None
    date = wb_filters.DateFilter(
        label=_("Week Day"),
        method=lambda queryset, label, value: queryset,
        default=monday_of_current_week,
        required=True,
        help_text="Change this date to any day on the week you are interested in seeing the presence table",
    )



# Generated by Django 4.1.8 on 2023-04-17 11:31
import colorfield.fields
import django.contrib.postgres.constraints
import django.contrib.postgres.fields.ranges
import django.core.validators
import django.db.models.deletion
import django_fsm
import mptt.fields
import timezone_field.fields
import wbcore.contrib.color.fields
import wbcore.contrib.icons.models
import wbhuman_resources.models.kpi
from django.contrib.postgres.operations import BtreeGistExtension
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
        ("agenda", "0003_calendaritem_endpoint_basename"),
        ("directory", "0002_auto_20230414_1553"),
    ]

    operations = [
        BtreeGistExtension(),
        migrations.CreateModel(
            name="AbsenceRequest",
            fields=[
                (
                    "calendaritem_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="agenda.calendaritem",
                    ),
                ),
                (
                    "status",
                    django_fsm.FSMField(
                        choices=[
                            ("DRAFT", "Draft"),
                            ("PENDING", "Pending"),
                            ("APPROVED", "Approved"),
                            ("DENIED", "Denied"),
                            ("CANCELLED", "Cancelled"),
                        ],
                        default="DRAFT",
                        help_text="The request status (defaults to draft)",
                        max_length=50,
                        verbose_name="Status",
                    ),
                ),
                (
                    "notes",
                    models.TextField(
                        blank=True, help_text="A note to the HR administrator", null=True, verbose_name="Extra Notes"
                    ),
                ),
                (
                    "reason",
                    models.TextField(
                        blank=True,
                        help_text="The HR's response to this absence request",
                        null=True,
                        verbose_name="Reason",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        auto_now_add=True, help_text="The request creation time", verbose_name="Created"
                    ),
                ),
                (
                    "attachment",
                    models.FileField(
                        blank=True,
                        help_text="Upload a file to document this absence request (e.g. medical certificate)",
                        max_length=256,
                        null=True,
                        upload_to="human_resources/absence_request/attachments",
                        verbose_name="Attachment",
                    ),
                ),
            ],
            options={
                "verbose_name": "Absence Request",
                "verbose_name_plural": "Absence Requests",
                "permissions": [("administrate_absencerequest", "Can administer Absence Requests")],
            },
            bases=("agenda.calendaritem",),
        ),
        migrations.CreateModel(
            name="AbsenceRequestType",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("color", wbcore.contrib.color.fields.ColorField(default="#000000", verbose_name="Color")),
                ("icon", wbcore.contrib.icons.models.IconField(default="EVENT", max_length=128, verbose_name="Icon")),
                ("title", models.CharField(max_length=255, verbose_name="Title")),
                (
                    "is_vacation",
                    models.BooleanField(
                        default=False,
                        help_text="If true, the days will be counted towards the employee's vacation balance",
                        verbose_name="Vacation",
                    ),
                ),
                (
                    "is_timeoff",
                    models.BooleanField(
                        default=False,
                        help_text="If true, the employee is considered as not working",
                        verbose_name="Time-Off",
                    ),
                ),
                (
                    "is_extensible",
                    models.BooleanField(
                        default=False,
                        help_text="If true, allow the associated request to be extended",
                        verbose_name="Extensible",
                    ),
                ),
                ("auto_approve", models.BooleanField(default=False, verbose_name="Auto Approve")),
                ("days_in_advance", models.PositiveIntegerField(default=0, verbose_name="Days In Advance")),
            ],
            options={
                "verbose_name": "Absence Request Type",
                "verbose_name_plural": "Absence Request Types",
            },
        ),
        migrations.CreateModel(
            name="DayOffCalendar",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("title", models.CharField(max_length=255)),
                (
                    "resource",
                    models.CharField(
                        blank=True, help_text="Used to fetch the days off from an API.", max_length=255, null=True
                    ),
                ),
                ("timezone", timezone_field.fields.TimeZoneField(default="UTC")),
            ],
            options={
                "verbose_name": "Day Off Calendar",
                "verbose_name_plural": "Days Off Calendar",
                "unique_together": {("resource", "timezone")},
            },
        ),
        migrations.CreateModel(
            name="DefaultDailyPeriod",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("lower_time", models.TimeField()),
                ("upper_time", models.TimeField()),
                ("timespan", django.contrib.postgres.fields.ranges.DateTimeRangeField()),
                ("title", models.CharField(max_length=128)),
                ("total_hours", models.FloatField()),
                (
                    "calendar",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="default_periods",
                        to="wbhuman_resources.dayoffcalendar",
                        verbose_name="Calendar",
                    ),
                ),
            ],
            options={
                "verbose_name": "Default Daily Period",
                "verbose_name_plural": "Default Daily Periods",
            },
        ),
        migrations.CreateModel(
            name="EmployeeHumanResource",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("computed_str", models.CharField(blank=True, max_length=512, null=True, verbose_name="Name")),
                (
                    "is_active",
                    models.BooleanField(
                        default=True,
                        help_text="If false, the employee will be considered as not active but his absence requests will be preserved",
                        verbose_name="Is active",
                    ),
                ),
                (
                    "extra_days_frequency",
                    models.CharField(
                        choices=[("MONTHLY", "Monthly"), ("YEARLY", "Yearly")],
                        default="YEARLY",
                        help_text="The frequency at which an additional number of vacation days is enabled for this employee (defaults to yearly)",
                        max_length=16,
                        verbose_name="Extra Days Frequency",
                    ),
                ),
                (
                    "occupancy_rate",
                    models.FloatField(
                        default=1,
                        help_text="The occupation rate in percent, 100% being employed fulltime",
                        verbose_name="Occupation Rate",
                    ),
                ),
                (
                    "contract_type",
                    models.CharField(
                        choices=[("INTERNAL", "Internal"), ("EXTERNAL", "External")],
                        default="INTERNAL",
                        help_text="If Internal, the employee is considered a full-time employee and thus has employee access to the system.",
                        max_length=16,
                        verbose_name="Employee Type",
                    ),
                ),
                ("enrollment_at", models.DateField(verbose_name="Enrolled at")),
                ("disenrollment_at", models.DateField(blank=True, null=True, verbose_name="Disenroll at")),
                (
                    "calendar",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="employees",
                        to="wbhuman_resources.dayoffcalendar",
                    ),
                ),
                (
                    "direct_manager",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="managed_employees",
                        to="directory.person",
                        verbose_name="Direct Manager",
                    ),
                ),
            ],
            options={
                "verbose_name": "Employee Human Resource",
                "verbose_name_plural": "Employee Human Resources",
            },
        ),
        migrations.CreateModel(
            name="Evaluation",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("evaluated_score", models.IntegerField(blank=True, null=True)),
                ("evaluated_period", django.contrib.postgres.fields.ranges.DateRangeField()),
                ("evaluation_date", models.DateField()),
                (
                    "last_update",
                    models.DateTimeField(
                        auto_now=True,
                        help_text="Date of latest change (automatically computed)",
                        verbose_name="Last update",
                    ),
                ),
            ],
            options={
                "verbose_name": "Evaluation",
                "verbose_name_plural": "Evaluations",
            },
        ),
        migrations.CreateModel(
            name="Review",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("computed_str", models.CharField(blank=True, max_length=512, null=True, verbose_name="Name")),
                ("from_date", models.DateField(blank=True, null=True, verbose_name="From")),
                ("to_date", models.DateField(blank=True, null=True, verbose_name="To")),
                ("review_deadline", models.DateField(blank=True, null=True, verbose_name="Deadline")),
                ("review", models.DateTimeField(blank=True, null=True, verbose_name="Review Date")),
                ("auto_apply_deadline", models.BooleanField(default=True, verbose_name="Auto Apply Deadline")),
                (
                    "year",
                    models.IntegerField(
                        blank=True,
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(1000),
                            django.core.validators.MaxValueValidator(9999),
                        ],
                        verbose_name="Year",
                    ),
                ),
                (
                    "type",
                    models.CharField(
                        choices=[("ANNUAL", "Annual"), ("INTERMEDIARY", "Intermediary")],
                        default="ANNUAL",
                        max_length=32,
                        verbose_name="Type",
                    ),
                ),
                (
                    "status",
                    django_fsm.FSMField(
                        choices=[
                            ("PREPARATION_OF_REVIEW", "Stage 1: Preparation of review"),
                            ("FILL_IN_REVIEW", "Stage 2: Fill in review"),
                            ("REVIEW", "Stage 3: Review"),
                            ("EVALUATION", "Stage 4: Evalutation"),
                            ("VALIDATION", "Stage 5: Validation"),
                        ],
                        default="PREPARATION_OF_REVIEW",
                        help_text="Indicates one of the four stages defined by the workflow",
                        max_length=50,
                        verbose_name="Status",
                    ),
                ),
                ("is_template", models.BooleanField(default=False)),
                ("feedback_reviewee", models.TextField(blank=True, default="", verbose_name="Feedback Reviewee")),
                ("feedback_reviewer", models.TextField(blank=True, default="", verbose_name="Feedback Reviewer")),
                ("editable_mode", models.BooleanField(default=True, verbose_name="Editable table mode")),
                (
                    "signed_reviewee",
                    models.DateTimeField(blank=True, null=True, verbose_name="Date of reviewee's signature"),
                ),
                (
                    "signed_reviewer",
                    models.DateTimeField(blank=True, null=True, verbose_name="Date of reviewer's signature"),
                ),
                (
                    "completely_filled_reviewee",
                    models.DateTimeField(blank=True, null=True, verbose_name="Completely Filled Out Reviewee"),
                ),
                (
                    "completely_filled_reviewer",
                    models.DateTimeField(blank=True, null=True, verbose_name="Completely Filled Out Reviewer"),
                ),
                ("changed", models.DateTimeField(auto_now=True, null=True)),
                (
                    "moderator",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="moderator_reviews",
                        to="directory.person",
                    ),
                ),
            ],
            options={
                "verbose_name": "Review",
                "verbose_name_plural": "Reviews",
                "permissions": [
                    ("admin_review", "Can administer Reviews"),
                    ("global_review_readers", "Can view and read Review"),
                ],
            },
        ),
        migrations.CreateModel(
            name="ReviewQuestionCategory",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(max_length=255, unique=True, verbose_name="Name")),
                ("order", models.PositiveIntegerField(blank=True, null=True, verbose_name="Order")),
                ("weight", models.DecimalField(decimal_places=1, default=1.0, max_digits=16, verbose_name="Weight")),
            ],
            options={
                "verbose_name": "Review Question Category",
                "verbose_name_plural": "Review Question Categories",
            },
        ),
        migrations.CreateModel(
            name="ReviewQuestion",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("computed_str", models.CharField(blank=True, max_length=512, null=True, verbose_name="Name")),
                ("question", models.TextField(blank=True, default="", verbose_name="Question")),
                ("mandatory", models.BooleanField(default=True, verbose_name="Mandatory")),
                (
                    "answer_type",
                    models.CharField(
                        choices=[("TEXT", "Text"), ("RATING", "Rating")],
                        default="TEXT",
                        max_length=32,
                        verbose_name="Type",
                    ),
                ),
                ("for_reviewee", models.BooleanField(default=True, verbose_name="For Reviewee")),
                ("for_reviewer", models.BooleanField(default=True, verbose_name="For Reviewer")),
                ("for_department_peers", models.BooleanField(default=False, verbose_name="For Department Peers")),
                ("for_company_peers", models.BooleanField(default=False, verbose_name="For Company Peers")),
                ("order", models.PositiveIntegerField(blank=True, null=True, verbose_name="Order")),
                ("weight", models.DecimalField(decimal_places=1, default=1.0, max_digits=16, verbose_name="Weight")),
                (
                    "category",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="questions_related",
                        to="wbhuman_resources.reviewquestioncategory",
                        verbose_name="Category",
                    ),
                ),
                (
                    "evaluation",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="evaluation_questions",
                        to="wbhuman_resources.evaluation",
                        verbose_name="Evaluation",
                    ),
                ),
                (
                    "review",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="questions",
                        to="wbhuman_resources.review",
                        verbose_name="Review",
                    ),
                ),
            ],
            options={
                "verbose_name": "Review Question",
                "verbose_name_plural": "Review Questions",
            },
        ),
        migrations.CreateModel(
            name="ReviewGroup",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(max_length=255, verbose_name="Name")),
                ("employees", models.ManyToManyField(blank=True, related_name="reviewgroups", to="directory.person")),
            ],
            options={
                "verbose_name": "Review Group",
                "verbose_name_plural": "Review Groups",
            },
        ),
        migrations.CreateModel(
            name="ReviewAnswer",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "answered_anonymized",
                    models.CharField(blank=True, max_length=255, null=True, verbose_name="Answered Anonymized"),
                ),
                (
                    "answer_number",
                    models.PositiveIntegerField(
                        blank=True,
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(1),
                            django.core.validators.MaxValueValidator(4),
                        ],
                        verbose_name="Rating",
                    ),
                ),
                ("answer_text", models.TextField(blank=True, null=True, verbose_name="Comment")),
                (
                    "answered_by",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="related_answers",
                        to="directory.person",
                        verbose_name="Answered By",
                    ),
                ),
                (
                    "question",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="answers",
                        to="wbhuman_resources.reviewquestion",
                        verbose_name="Question",
                    ),
                ),
            ],
            options={
                "verbose_name": "Review Answer",
                "verbose_name_plural": "Review Answers",
            },
        ),
        migrations.AddField(
            model_name="review",
            name="review_group",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="review_related",
                to="wbhuman_resources.reviewgroup",
                verbose_name="Group",
            ),
        ),
        migrations.AddField(
            model_name="review",
            name="reviewee",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="reviewee_reviews",
                to="directory.person",
                verbose_name="Reviewee",
            ),
        ),
        migrations.AddField(
            model_name="review",
            name="reviewer",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="reviewer_reviews",
                to="directory.person",
                verbose_name="Reviewer",
            ),
        ),
        migrations.CreateModel(
            name="Position",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("computed_str", models.CharField(blank=True, max_length=512, null=True, verbose_name="Name")),
                (
                    "color",
                    colorfield.fields.ColorField(default="#FF0000", image_field=None, max_length=18, samples=None),
                ),
                ("name", models.CharField(max_length=256)),
                ("height", models.IntegerField(default=0)),
                ("is_internal", models.BooleanField(default=True, verbose_name="Internal Position")),
                ("lft", models.PositiveIntegerField(editable=False)),
                ("rght", models.PositiveIntegerField(editable=False)),
                ("tree_id", models.PositiveIntegerField(db_index=True, editable=False)),
                ("level", models.PositiveIntegerField(editable=False)),
                (
                    "group",
                    models.OneToOneField(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="human_resources_positions",
                        to="auth.group",
                    ),
                ),
                (
                    "manager",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="managed_positions",
                        to="directory.person",
                        verbose_name="Department Manager",
                    ),
                ),
                (
                    "parent",
                    mptt.fields.TreeForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="children",
                        to="wbhuman_resources.position",
                        verbose_name="Parent Positions",
                    ),
                ),
            ],
            options={
                "verbose_name": "Position",
                "verbose_name_plural": "Positions",
            },
        ),
        migrations.CreateModel(
            name="KPI",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(max_length=255)),
                ("goal", models.PositiveIntegerField(verbose_name="Goal")),
                ("period", django.contrib.postgres.fields.ranges.DateRangeField(verbose_name="Period")),
                (
                    "evaluated_intervals",
                    models.CharField(
                        choices=[
                            ("DAILY", "Daily"),
                            ("WEEKLY", "Weekly"),
                            ("MONTHLY", "Monthly"),
                            ("QUARTERLY", "Quarterly"),
                        ],
                        default="MONTHLY",
                        max_length=16,
                        verbose_name="Evaluated Intervals",
                    ),
                ),
                ("handler", models.CharField(max_length=255)),
                ("additional_data", models.JSONField(default=wbhuman_resources.models.kpi.default_additional_data)),
                (
                    "last_update",
                    models.DateTimeField(
                        auto_now=True,
                        help_text="Date of latest change (automatically computed)",
                        verbose_name="Last update",
                    ),
                ),
                ("individual_evaluation", models.BooleanField(default=True, verbose_name="Individual Evaluation")),
                ("is_active", models.BooleanField(default=True, verbose_name="Is Active")),
                (
                    "evaluated_persons",
                    models.ManyToManyField(
                        related_name="wbhuman_resources_kpis", to="directory.person", verbose_name="Evaluated Persons"
                    ),
                ),
            ],
            options={
                "verbose_name": "Key Performance Indicator",
                "verbose_name_plural": "Key Performance Indicators",
                "permissions": [("admin_kpi", "Can administer KPI")],
            },
        ),
        migrations.AddField(
            model_name="evaluation",
            name="kpi",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="evaluations",
                to="wbhuman_resources.kpi",
                verbose_name="KPI",
            ),
        ),
        migrations.AddField(
            model_name="evaluation",
            name="person",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="wbhuman_resources_evaluations",
                to="directory.person",
            ),
        ),
        migrations.CreateModel(
            name="EmployeeYearBalance",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("computed_str", models.CharField(blank=True, max_length=512, null=True, verbose_name="Name")),
                ("year", models.IntegerField(verbose_name="Year")),
                (
                    "extra_balance",
                    models.FloatField(
                        default=0,
                        help_text="The yearly extra balance (in hours)",
                        verbose_name="Extra Balance (in hours)",
                    ),
                ),
                (
                    "employee",
                    models.ForeignKey(
                        help_text="The employee having that year balance",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="balances",
                        to="wbhuman_resources.employeehumanresource",
                        verbose_name="Employee",
                    ),
                ),
            ],
            options={
                "verbose_name": "Employee Year Balance",
                "verbose_name_plural": "Employee Year Balances",
            },
        ),
        migrations.CreateModel(
            name="EmployeeWeeklyOffPeriods",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("computed_str", models.CharField(blank=True, max_length=512, null=True, verbose_name="Name")),
                (
                    "weekday",
                    models.PositiveIntegerField(
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(6),
                        ]
                    ),
                ),
                (
                    "employee",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="default_periods_relationships",
                        to="wbhuman_resources.employeehumanresource",
                        verbose_name="Employee",
                    ),
                ),
                (
                    "period",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="employees_relationships",
                        to="wbhuman_resources.defaultdailyperiod",
                        verbose_name="Off Period",
                    ),
                ),
            ],
            options={
                "verbose_name": "Employee Weekly off Period",
                "verbose_name_plural": "Employee Weekly off Periods",
            },
        ),
        migrations.AddField(
            model_name="employeehumanresource",
            name="position",
            field=models.ForeignKey(
                blank=True,
                help_text="The position this employee belongs to",
                limit_choices_to=models.Q(("height", 0)),
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="employees",
                to="wbhuman_resources.position",
                verbose_name="Position",
            ),
        ),
        migrations.AddField(
            model_name="employeehumanresource",
            name="profile",
            field=models.OneToOneField(
                help_text="The CRM profile related to this employee",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="human_resources",
                to="directory.person",
                verbose_name="Employee",
            ),
        ),
        migrations.AddField(
            model_name="employeehumanresource",
            name="weekly_off_periods",
            field=models.ManyToManyField(
                related_name="employees_off",
                through="wbhuman_resources.EmployeeWeeklyOffPeriods",
                to="wbhuman_resources.defaultdailyperiod",
                verbose_name="Weekly off periods",
            ),
        ),
        migrations.CreateModel(
            name="DayOff",
            fields=[
                (
                    "calendaritem_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="agenda.calendaritem",
                    ),
                ),
                ("date", models.DateField(verbose_name="Date")),
                (
                    "count_as_holiday",
                    models.BooleanField(
                        default=True,
                        help_text="If true, there is no work but the day counts towards the employees' vacation balance",
                        verbose_name="Count as Holiday",
                    ),
                ),
                (
                    "calendar",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="days_off",
                        to="wbhuman_resources.dayoffcalendar",
                        verbose_name="Calendar",
                    ),
                ),
            ],
            options={
                "verbose_name": "Day Off",
                "verbose_name_plural": "Days Off",
            },
            bases=("agenda.calendaritem",),
        ),
        migrations.CreateModel(
            name="BalanceHourlyAllowance",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("period_index", models.PositiveIntegerField()),
                ("hourly_allowance", models.FloatField()),
                (
                    "balance",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="monthly_allowances",
                        to="wbhuman_resources.employeeyearbalance",
                        verbose_name="Balance",
                    ),
                ),
            ],
            options={
                "verbose_name": "Monthly Allowance",
                "verbose_name_plural": "Monthly Allowance",
            },
        ),
        migrations.CreateModel(
            name="AbsenceRequestPeriods",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("date", models.DateField()),
                ("timespan", django.contrib.postgres.fields.ranges.DateTimeRangeField(verbose_name="Timespan")),
                (
                    "consecutive_hours_count",
                    models.IntegerField(
                        default=0,
                        help_text="The number of consecutive hours this absence request period spans",
                        verbose_name="Consecutive Absence Hours count",
                    ),
                ),
                (
                    "balance",
                    models.ForeignKey(
                        blank=True,
                        help_text="For which balance this absence will count towards",
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="periods",
                        to="wbhuman_resources.employeeyearbalance",
                        verbose_name="Balance",
                    ),
                ),
                (
                    "default_period",
                    models.ForeignKey(
                        help_text="The associated period",
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="periods",
                        to="wbhuman_resources.defaultdailyperiod",
                        verbose_name="Period",
                    ),
                ),
                (
                    "employee",
                    models.ForeignKey(
                        help_text="The Requester",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="periods",
                        to="wbhuman_resources.employeehumanresource",
                        verbose_name="Employee",
                    ),
                ),
                (
                    "request",
                    models.ForeignKey(
                        help_text="The associated request",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="periods",
                        to="wbhuman_resources.absencerequest",
                        verbose_name="Request",
                    ),
                ),
            ],
            options={
                "verbose_name": "Absence Request Period",
                "verbose_name_plural": "Absence Request Periods",
            },
        ),
        migrations.AddField(
            model_name="absencerequest",
            name="employee",
            field=models.ForeignKey(
                help_text="The employee requesting the absence",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="requests",
                to="wbhuman_resources.employeehumanresource",
                verbose_name="Employee",
            ),
        ),
        migrations.AddField(
            model_name="absencerequest",
            name="type",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="request",
                to="wbhuman_resources.absencerequesttype",
                verbose_name="Type",
            ),
        ),
        migrations.AddIndex(
            model_name="employeeyearbalance",
            index=models.Index(fields=["employee", "year"], name="wbhuman_res_employe_b6cd2e_idx"),
        ),
        migrations.AlterUniqueTogether(
            name="employeeyearbalance",
            unique_together={("employee", "year")},
        ),
        migrations.AddIndex(
            model_name="employeeweeklyoffperiods",
            index=models.Index(fields=["employee", "period", "weekday"], name="wbhuman_res_employe_248df8_idx"),
        ),
        migrations.AlterUniqueTogether(
            name="employeeweeklyoffperiods",
            unique_together={("employee", "period", "weekday")},
        ),
        migrations.AddConstraint(
            model_name="defaultdailyperiod",
            constraint=models.CheckConstraint(
                check=models.Q(("upper_time__gt", models.F("lower_time"))), name="check_lower_time_lt_upper_time"
            ),
        ),
        migrations.AddConstraint(
            model_name="defaultdailyperiod",
            constraint=django.contrib.postgres.constraints.ExclusionConstraint(
                expressions=[("timespan", "&&"), ("calendar", "=")], name="check_no_overlapping_default_periods_time"
            ),
        ),
        migrations.AddConstraint(
            model_name="dayoff",
            constraint=models.UniqueConstraint(fields=("date", "calendar"), name="unique_date_for_calendar"),
        ),
        migrations.AddIndex(
            model_name="balancehourlyallowance",
            index=models.Index(fields=["balance"], name="wbhuman_res_balance_d67ba9_idx"),
        ),
        migrations.AddIndex(
            model_name="balancehourlyallowance",
            index=models.Index(fields=["balance", "period_index"], name="wbhuman_res_balance_35163a_idx"),
        ),
        migrations.AlterUniqueTogether(
            name="balancehourlyallowance",
            unique_together={("balance", "period_index")},
        ),
        migrations.AddIndex(
            model_name="absencerequestperiods",
            index=models.Index(fields=["employee", "default_period", "date"], name="wbhuman_res_employe_5eb827_idx"),
        ),
        migrations.AddConstraint(
            model_name="absencerequestperiods",
            constraint=django.contrib.postgres.constraints.ExclusionConstraint(
                expressions=[("timespan", "&&"), ("employee", "=")], name="exclude_overlapping_periods"
            ),
        ),
        migrations.AlterUniqueTogether(
            name="absencerequestperiods",
            unique_together={("employee", "default_period", "date")},
        ),
    ]



from .absence import (
    AbsenceRequestEmployeeHumanResourceFilterSet,
    AbsenceRequestFilter,
    AbsenceTypeCountEmployeeModelFilterSet,
)
from .absence_graphs import AbsenceRequestPlannerFilter, AbsenceTableFilter
from .calendars import DayOffFilter
from .employee import EmployeeBalanceFilterSet, EmployeeFilterSet, PositionFilterSet
from .kpi import KPIFilterSet, KPIEvaluationFilterSet, KPIEvaluationPandasFilter
from .review import (
    ReviewGroupFilter,
    ReviewTemplateFilter,
    ReviewFilter,
    ReviewQuestionCategoryFilter,
    ReviewQuestionFilter,
    ReviewAnswerFilter,
    ReviewProgressReviewFilter,
    RatingReviewAnswerReviewFilter,
)
from .signals import *



from datetime import datetime

from django.db.models import Q
from django.utils.translation import gettext
from django.utils.translation import gettext_lazy as _
from rest_framework import serializers as rf_serializers
from rest_framework.reverse import reverse
from wbcore import serializers as wb_serializers
from wbcore.contrib.directory.models import Person
from wbcore.contrib.directory.serializers import PersonRepresentationSerializer

from wbhuman_resources.models import (
    Review,
    ReviewAnswer,
    ReviewGroup,
    ReviewQuestion,
    ReviewQuestionCategory,
)


class ReviewGroupRepresentationSerializer(wb_serializers.RepresentationSerializer):
    _detail = wb_serializers.HyperlinkField(reverse_name="wbhuman_resources:reviewgroup-detail")

    class Meta:
        model = ReviewGroup
        fields = ("id", "name", "_detail")


class ReviewRepresentationSerializer(wb_serializers.RepresentationSerializer):
    id_repr = wb_serializers.CharField(source="id", read_only=True, label=_("ID"))
    _detail = wb_serializers.HyperlinkField(reverse_name="wbhuman_resources:review-detail")

    class Meta:
        model = Review
        fields = ("id", "id_repr", "computed_str", "_detail")


class ReviewQuestionCategoryRepresentationSerializer(wb_serializers.RepresentationSerializer):
    class Meta:
        model = ReviewQuestionCategory
        fields = (
            "id",
            "name",
        )


class ReviewQuestionRepresentationSerializer(wb_serializers.RepresentationSerializer):
    class Meta:
        model = ReviewQuestion
        fields = (
            "id",
            "computed_str",
            "question",
        )


class ReviewAnswerRepresentationSerializer(wb_serializers.RepresentationSerializer):
    class Meta:
        model = ReviewAnswer
        fields = ("id",)


class ReviewGroupModelSerializer(wb_serializers.ModelSerializer):
    @wb_serializers.register_resource()
    def register_history_resource(self, instance, request, user):
        resources = {
            "review": reverse("wbhuman_resources:reviewgroup-review-list", args=[instance.id], request=request),
        }

        if Review.objects.filter(
            Q(review_group=instance) & Q(status=Review.Status.PREPARATION_OF_REVIEW) & Q(moderator=user.profile)
        ):
            resources["submitreviews"] = reverse(
                "wbhuman_resources:reviewgroup-submitreviews", args=[instance.id], request=request
            )

        return resources

    _employees = PersonRepresentationSerializer(many=True, source="employees")

    class Meta:
        model = ReviewGroup
        fields = ("id", "name", "employees", "_employees", "_additional_resources")


class ReviewModelSerializer(wb_serializers.ModelSerializer):
    @wb_serializers.register_resource()
    def register_history_resource(self, instance, request, user):
        resources = {}
        categories = instance.get_question_categories()
        if instance.status == Review.Status.PREPARATION_OF_REVIEW:
            resources["category"] = reverse("wbhuman_resources:reviewquestioncategory-list", args=[], request=request)
            resources["questionnocategory"] = reverse(
                "wbhuman_resources:review-reviewquestionnocategory-list", args=[instance.id], request=request
            )
            for category in categories:
                resources[f"questioncategory{category.id}"] = reverse(
                    "wbhuman_resources:review-reviewquestioncategory-list",
                    args=[instance.id, category.id],
                    request=request,
                )
        else:
            resources["reviewanswerquestionnocategory"] = reverse(
                "wbhuman_resources:review-reviewanswerquestionnocategory-list", args=[instance.id], request=request
            )
            for category in categories:
                resources[f"reviewanswerquestioncategory{category.id}"] = reverse(
                    "wbhuman_resources:review-reviewanswerquestioncategory-list",
                    args=[instance.id, category.id],
                    request=request,
                )
            resources["progress"] = reverse(
                "wbhuman_resources:review-progress-list", args=[instance.id], request=request
            )

        if instance.is_template and instance.review_group and instance.moderator == user.profile:
            resources["generate"] = reverse("wbhuman_resources:review-generate", args=[instance.id], request=request)

        if instance.status == Review.Status.FILL_IN_REVIEW:
            if instance.reviewee == user.profile and not instance.completely_filled_reviewee:
                resources["completelyfilledreviewee"] = reverse(
                    "wbhuman_resources:review-completelyfilled-reviewee", args=[instance.id], request=request
                )
            if instance.reviewer == user.profile and not instance.completely_filled_reviewer:
                resources["completelyfilledreviewer"] = reverse(
                    "wbhuman_resources:review-completelyfilled-reviewer", args=[instance.id], request=request
                )

        if instance.status in [Review.Status.EVALUATION, Review.Status.VALIDATION]:
            resources["rating_review_answer_key"] = (
                reverse("wbhuman_resources:review-reviewanswerpandasview-list", args=[instance.id], request=request)
                + "?answer_type=RATING"
            )
            resources["text_review_answer_key"] = (
                reverse("wbhuman_resources:review-reviewanswerpandasview-list", args=[instance.id], request=request)
                + "?answer_type=TEXT"
            )

            if instance.status == Review.Status.EVALUATION:
                if instance.reviewee == user.profile and not instance.signed_reviewee:
                    resources["signaturereviewee"] = reverse(
                        "wbhuman_resources:review-signature-reviewee", args=[instance.id], request=request
                    )
                if instance.reviewer == user.profile and not instance.signed_reviewer:
                    resources["signaturereviewer"] = reverse(
                        "wbhuman_resources:review-signature-reviewer", args=[instance.id], request=request
                    )

            if instance.status == Review.Status.VALIDATION and (
                instance.moderator == user.profile or user.has_perm("wbhuman_resources.administrate_review")
            ):
                resources["generate_pdf"] = reverse(
                    "wbhuman_resources:review-generate-pdf", args=[instance.id], request=request
                )

        return resources

    _reviewee = PersonRepresentationSerializer(source="reviewee")
    _reviewer = PersonRepresentationSerializer(source="reviewer")
    moderator = wb_serializers.PrimaryKeyRelatedField(
        default=wb_serializers.CurrentUserDefault("profile"), queryset=Person.objects.all(), label=_("Moderator")
    )
    _moderator = PersonRepresentationSerializer(source="moderator")

    _review_group = ReviewGroupRepresentationSerializer(source="review_group")

    global_rating = wb_serializers.DecimalField(
        read_only=True,
        max_digits=14,
        decimal_places=2,
        help_text=_("Only the rating questions you answered are taken into account"),
    )
    year = wb_serializers.YearField(default=lambda: datetime.now().year)
    is_template = wb_serializers.BooleanField(default=True, label=_("Is Template"))

    reviewee = wb_serializers.PrimaryKeyRelatedField(
        default=wb_serializers.DefaultAttributeFromObject(source="reviewee"),
        read_only=lambda view: not view.is_modifiable,
        queryset=Person.objects.all(),
    )
    reviewer = wb_serializers.PrimaryKeyRelatedField(
        default=wb_serializers.DefaultAttributeFromObject(source="reviewer"),
        read_only=lambda view: not view.is_modifiable,
        queryset=Person.objects.all(),
    )

    class Meta:
        model = Review
        fields = (
            "id",
            "computed_str",
            "from_date",
            "to_date",
            "review_deadline",
            "review",
            "auto_apply_deadline",
            "status",
            "reviewee",
            "_reviewee",
            "reviewer",
            "_reviewer",
            "moderator",
            "_moderator",
            "review_group",
            "_review_group",
            "is_template",
            "feedback_reviewee",
            "feedback_reviewer",
            "_additional_resources",
            "signed_reviewee",
            "signed_reviewer",
            "year",
            "type",
            "completely_filled_reviewee",
            "completely_filled_reviewer",
            "global_rating",
            "changed",
        )
        read_only_fields = (
            "signed_reviewee",
            "signed_reviewer",
            "completely_filled_reviewee",
            "completely_filled_reviewer",
        )

    def validate(self, data):
        obj = self.instance
        errors = {}
        reviewer = data.get("reviewer", obj.reviewer if obj else None)
        reviewee = data.get("reviewee", obj.reviewee if obj else None)
        if data.get("is_template", obj.is_template if obj else None):
            dict_fields = {
                "from_date": data.get("from_date", obj.from_date if obj else None),
                "to_date": data.get("to_date", obj.to_date if obj else None),
                "review_deadline": data.get("review_deadline", obj.review_deadline if obj else None),
                "reviewee": data.get("reviewee", obj.reviewee if obj else None),
                "reviewer": data.get("reviewer", obj.reviewer if obj else None),
            }
            for key, value in dict_fields.items():
                if value:
                    errors[key] = [gettext("The field has to be empty if is_template is True")]
        elif reviewer == reviewee is not None:
            errors["reviewer"] = gettext("reviewer must be different from reviewee")
            errors["reviewee"] = gettext("reviewer must be different from reviewee")

        if len(errors.keys()) > 0:
            raise rf_serializers.ValidationError(errors)

        return super().validate(data)


class ReviewListModelSerializer(ReviewModelSerializer):
    class Meta:
        model = Review
        fields = (
            "id",
            "from_date",
            "to_date",
            "review_deadline",
            "review",
            "auto_apply_deadline",
            "status",
            "reviewee",
            "_reviewee",
            "reviewer",
            "_reviewer",
            "moderator",
            "_moderator",
            "review_group",
            "_review_group",
            "is_template",
            "year",
            "type",
            "changed",
            "_additional_resources",
        )


class ReviewReadOnlyModelSerializer(ReviewModelSerializer):
    @wb_serializers.register_resource()
    def register_history_resource(self, instance, request, user):
        return {}


class ReviewQuestionCategoryModelSerializer(wb_serializers.ModelSerializer):
    @wb_serializers.register_resource()
    def register_history_resource(self, instance, request, user):
        resources = {
            "reviewquestion": reverse(
                "wbhuman_resources:reviewquestioncategory-reviewquestion-list", args=[instance.id], request=request
            ),
        }
        return resources

    class Meta:
        model = ReviewQuestionCategory
        fields = ("id", "name", "order", "weight", "_additional_resources")


class ReviewQuestionModelSerializer(wb_serializers.ModelSerializer):
    _review = ReviewRepresentationSerializer(source="review")
    category = wb_serializers.PrimaryKeyRelatedField(
        default=wb_serializers.DefaultFromKwargs("category_id"), queryset=ReviewQuestionCategory.objects.all()
    )
    _category = ReviewQuestionCategoryRepresentationSerializer(source="category")

    review_for = wb_serializers.CharField(label=_("Review For"), read_only=True)
    question = wb_serializers.TextAreaField(label=_("Question"), allow_blank=True)

    @wb_serializers.register_resource()
    def register_resource(self, instance, request, user):
        resources = {
            "review_answers_table": reverse("wbhuman_resources:reviewanswer-list", args=[], request=request)
            + f"?question={instance.id}",
        }
        return resources

    class Meta:
        model = ReviewQuestion
        fields = (
            "id",
            "review",
            "_review",
            "category",
            "_category",
            "question",
            "computed_str",
            "mandatory",
            "answer_type",
            "for_reviewee",
            "for_reviewer",
            "for_department_peers",
            "for_company_peers",
            "order",
            "weight",
            "review_for",
            "_additional_resources",
        )

    def validate(self, data):
        obj = self.instance
        for_department_peers = data.get("for_department_peers", obj.for_department_peers if obj else None)
        for_company_peers = data.get("for_company_peers", obj.for_company_peers if obj else None)
        errors = {}
        if not obj:
            qs = ReviewQuestion.objects.filter(
                Q(review=data.get("review"))
                & Q(category=data.get("category"))
                & ~Q(answer_type=data.get("answer_type"))
            )
            if qs.exists():
                errors["answer_type"] = gettext("Questions in a section must be of the same type")

        if for_department_peers and for_company_peers:
            errors["non_field_errors"] = [
                gettext(
                    "Both 'For department peers' and 'For company peers' true does not make any sense. Only 1 can be active"
                )
            ]

        if len(errors.keys()) > 0:
            raise rf_serializers.ValidationError(errors)

        return super().validate(data)


class ReviewAnswerModelSerializer(wb_serializers.ModelSerializer):
    _question = ReviewQuestionRepresentationSerializer(source="question")
    _answered_by = PersonRepresentationSerializer(source="answered_by")
    answer_number = wb_serializers.EmojiRatingField(label=_("Rating"))
    answer_text = wb_serializers.TextAreaField(label=_("Comment"), allow_blank=True, allow_null=True)
    weight = wb_serializers.DecimalField(read_only=True, max_digits=16, decimal_places=1)
    mandatory = wb_serializers.BooleanField(
        label=_("Mandatory"),
        read_only=True,
    )

    question_name = wb_serializers.TextAreaField(read_only=True, label=_("Question"))

    class Meta:
        model = ReviewAnswer
        fields = (
            "id",
            "question",
            "_question",
            "answered_by",
            "_answered_by",
            "answered_anonymized",
            "answer_number",
            "answer_text",
            "weight",
            "question_name",
            "mandatory",
        )
        read_only_fields = ("question", "answered_by", "answered_anonymized")

    def validate(self, data):
        errors = {}
        if obj := self.instance:
            if obj.question.mandatory:
                answer_number = data.get("answer_number", obj.answer_number if obj else None)
                answer_text = data.get("answer_text", obj.answer_text if obj else None)
                if not answer_text and obj.question.answer_type == ReviewQuestion.ANSWERTYPE.TEXT:
                    errors["answer_text"] = gettext(
                        "Comment cannot be empty, a response to this question is mandatory"
                    )
                if not answer_number and obj.question.answer_type == ReviewQuestion.ANSWERTYPE.RATING:
                    errors["answer_number"] = gettext(
                        "Rating cannot be empty, a response to this question is mandatory"
                    )

        if len(errors.keys()) > 0:
            raise rf_serializers.ValidationError(errors)

        return super().validate(data)



from rest_framework.reverse import reverse
from wbcore.metadata.configs.endpoints import EndpointViewConfig

from wbhuman_resources.models import KPI


class KPIEndpointConfig(EndpointViewConfig):
    def get_endpoint(self, **kwargs):
        return super().get_endpoint()

    def get_instance_endpoint(self, **kwargs):
        if self.instance:
            if not KPI.is_administrator(self.request.user):
                return None
        return super().get_instance_endpoint()

    def get_create_endpoint(self, **kwargs):
        if KPI.is_administrator(self.request.user):
            return super().get_instance_endpoint()
        return None

    def get_delete_endpoint(self, **kwargs):
        if KPI.is_administrator(self.request.user):
            return super().get_delete_endpoint()
        return None


class KPIEvaluationEndpointConfig(EndpointViewConfig):
    def get_endpoint(self, **kwargs):
        return reverse(
            "wbhuman_resources:kpi-evaluation-list",
            args=[self.view.kwargs["kpi_id"]],
            request=self.request,
        )

    def get_instance_endpoint(self, **kwargs):
        if self.instance:
            return None
        return super().get_instance_endpoint()

    def get_create_endpoint(self, **kwargs):
        return None

    def get_delete_endpoint(self, **kwargs):
        return None


class EvaluationGraphEndpointConfig(EndpointViewConfig):
    def get_list_endpoint(self, **kwargs):
        return reverse(
            "wbhuman_resources:kpi-evaluationgraph-list",
            args=[self.view.kwargs["kpi_id"]],
            request=self.request,
        )


class KPIEvaluationPandasEndpointConfig(EndpointViewConfig):
    def get_list_endpoint(self, **kwargs):
        return reverse("wbhuman_resources:kpievaluationpandas-list", [], request=self.request)
